import React, { useState, useEffect } from 'react'
import { withClient, Client } from 'cozy-client'
import { translate } from 'cozy-ui/react/I18n'

import IFluidConfig from 'services/IFluidConfig'

import StyledIconButton from 'components/CommonKit/IconButton/StyledIconButton'
import StyledButton from 'components/CommonKit/Button/StyledButton'
import TrailingIcon from 'assets/icons/ico/trailing-icon.svg'
import { ConnectionService } from 'services/connectionService'
import { AccountService } from 'services/accountService'
import { Account, AuthLoginData, Trigger } from 'doctypes'

interface KonnectorLoginFormProps {
  fluidConfig: IFluidConfig
  onSuccess: Function
  account: Account
  trigger: Trigger
  client: Client
  t: Function
}

const KonnectorLoginForm: React.FC<KonnectorLoginFormProps> = ({
  fluidConfig,
  onSuccess,
  account,
  trigger,
  client,
  t,
}: KonnectorLoginFormProps) => {
  const konnectorName: string = fluidConfig.konnectorConfig.name
  const konnectorType: string = fluidConfig.konnectorConfig.type
  const siteLink: string = fluidConfig.siteLink

  const [login, setLogin] = useState<string>('')
  const [password, setPassword] = useState<string>('')
  const [error, setError] = useState<string>('')
  const [loading, setLoading] = useState<boolean>(false)

  function revealPassword(idInput: string) {
    const input = document.getElementById(idInput)
    if (input) {
      if (input.getAttribute('type') === 'password') {
        input.setAttribute('type', 'text')
      } else {
        input.setAttribute('type', 'password')
      }
    }
  }

  const connect = async () => {
    const connectionService = new ConnectionService(
      client,
      fluidConfig.konnectorConfig.slug,
      login,
      password
    )
    const { account, trigger } = await connectionService.connectNewUser()
    if (!trigger) {
      setError(t('KONNECTORCONFIG.ERROR_ACCOUNT_CREATION'))
      setLoading(false)
      return null
    }
    onSuccess(account, trigger)
  }

  const update = async () => {
    const auth = {
      login: login,
      password: password,
    }
    account.auth = auth
    const updatedAccount = await AccountService.updateAccount(client, account)
    onSuccess(updatedAccount, trigger)
  }

  const handleSubmit = async (e: React.FormEvent<HTMLFormElement>) => {
    e.preventDefault()
    try {
      setError('')
      setLoading(true)
      if (!login || !password) {
        setError(t('KONNECTORCONFIG.ERROR_NO_LOGIN_PASSWORD'))
        setLoading(false)
        return null
      }
      if (!account) {
        await connect()
      } else {
        await update()
      }
    } catch (error) {
      setLoading(false)
    }
  }

  useEffect(() => {
    if (account) {
      const auth: AuthLoginData = account.auth
      if (auth.login) {
        setLogin(auth.login)
      }
      setError(t('KONNECTORCONFIG.ERROR_LOGIN_FAILED'))
    }
  }, [])

  return (
    <form
      className="form"
      onSubmit={(e: React.FormEvent<HTMLFormElement>) => handleSubmit(e)}
    >
      {t('KONNECTORCONFIG.LABEL_FILLIN')} {konnectorName}
      <div className="form-group">
        <input
          id={'idFieldLogin' + konnectorType}
          type="text"
          className="form-control form-input"
          aria-describedby="emailHelp"
          placeholder="Adresse e-mail"
          name="login"
          onChange={(e: React.ChangeEvent<HTMLInputElement>) =>
            setLogin(e.target.value)
          }
          value={login}
        ></input>
      </div>
      <div className="form-group">
        <input
          id={'idFieldPassword' + konnectorType}
          type="password"
          className="form-control form-input"
          aria-describedby="PasswordHelp"
          placeholder="Mot de passe"
          name="password"
          onChange={(e: React.ChangeEvent<HTMLInputElement>) =>
            setPassword(e.target.value)
          }
          value={password}
        />
        <span>
          <StyledIconButton
            icon={TrailingIcon}
            className="form-trailing-icon"
            size={22}
            onClick={() => revealPassword('idFieldPassword' + konnectorType)}
          />
        </span>
        <div className="form-message">{error}</div>
      </div>
      <StyledButton
        className="form-button"
        type="submit"
        color="primary"
        disabled={loading}
      >
        {t('KONNECTORCONFIG.BTN_CONNECTION')}
      </StyledButton>
      <StyledButton
        type="button"
        color="secondary"
        disabled={loading}
        onClick={() => window.open(siteLink, '_blank')}
      >
        {t('KONNECTORCONFIG.BTN_NOACCOUNT')}
      </StyledButton>
    </form>
  )
}

export default translate()(withClient(KonnectorLoginForm))
